"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.addEslint = void 0;
const addScriptToPackage_1 = require("../helpers/addScriptToPackage");
const addToGitIgnore_1 = require("../helpers/addToGitIgnore");
const commitWithAuthor_1 = require("../helpers/commitWithAuthor");
const installPackage_1 = require("../helpers/installPackage");
const withStateLogger_1 = require("../helpers/withStateLogger");
const writeAndAddFile_1 = require("../helpers/writeAndAddFile");
exports.addEslint = (0, withStateLogger_1.withStateLogger)({ id: "eslint" }, (config) => __awaiter(void 0, void 0, void 0, function* () {
    yield (0, installPackage_1.installPackage)(config, [
        "eslint@8.23.1",
        "@types/eslint@8.4.6",
        "@typescript-eslint/eslint-plugin@5.37.0",
        "@typescript-eslint/parser@5.37.0",
        "eslint-plugin-import@2.26.0",
        "eslint-import-resolver-typescript@3.5.1",
    ]);
    const eslintRcObject = {
        extends: ["eslint:recommended", "plugin:import/recommended", "plugin:jest/recommended"],
        plugins: ["import"],
        rules: {
            "no-undef": "off",
            "no-unused-vars": [
                "off",
                {
                    ignoreRestSiblings: true,
                },
            ],
            "import/no-duplicates": [
                "error",
                {
                    considerQueryString: true,
                },
            ],
            "import/named": "off",
            "import/first": "error",
            "import/no-namespace": "error",
            "import/extensions": [
                "error",
                "always",
                {
                    js: "never",
                    jsx: "never",
                    ts: "never",
                    tsx: "never",
                },
            ],
            "import/order": [
                "error",
                {
                    groups: [
                        ["internal", "external", "builtin"],
                        ["parent", "sibling", "index"],
                    ],
                    alphabetize: {
                        order: "asc",
                        caseInsensitive: true,
                    },
                },
            ],
            "import/newline-after-import": "error",
            "import/no-anonymous-default-export": "error",
            "import/no-dynamic-require": "error",
            "import/no-self-import": "error",
            "import/no-useless-path-segments": [
                "error",
                {
                    noUselessIndex: true,
                },
            ],
            "import/no-relative-packages": "error",
            "import/no-unused-modules": "error",
            "import/no-deprecated": "error",
            "import/no-commonjs": "error",
            "import/no-amd": "error",
            "import/no-mutable-exports": "error",
            "import/no-unassigned-import": "error",
            "jest/expect-expect": "off",
        },
        settings: {
            "import/external-module-folders": ["node_modules", "node_modules/@types"],
            "import/parsers": {
                "@typescript-eslint/parser": [".ts", ".tsx"],
            },
            "import/resolver": {
                typescript: {
                    alwaysTryTypes: true,
                    project: "<root>/tsconfig.json",
                },
                node: true,
            },
        },
        overrides: [
            {
                files: ["src/**/*.ts", "src/**/*.tsx"],
                extends: [
                    "eslint:recommended",
                    "plugin:@typescript-eslint/eslint-recommended",
                    "plugin:@typescript-eslint/recommended",
                    "plugin:import/recommended",
                    "plugin:jest/recommended",
                ],
                plugins: ["import", "@typescript-eslint"],
                rules: {
                    "@typescript-eslint/explicit-module-boundary-types": "off",
                    "@typescript-eslint/no-unused-vars": [
                        "error",
                        {
                            ignoreRestSiblings: true,
                        },
                    ],
                    "@typescript-eslint/switch-exhaustiveness-check": "error",
                    "@typescript-eslint/no-empty-function": "off",
                },
                parserOptions: {
                    parser: "@typescript-eslint/parser",
                    ecmaVersion: 2018,
                    sourceType: "module",
                    project: "./tsconfig.json",
                },
            },
        ],
    };
    yield (0, writeAndAddFile_1.writeAndAddFile)(config, ".eslintrc.json", JSON.stringify(eslintRcObject, null, 2));
    yield (0, addScriptToPackage_1.appendScriptToPackage)(config, "lint", "eslint --cache --ignore-path .gitignore --ext ts,js,tsx,jsx .");
    yield (0, addToGitIgnore_1.addToGitIgnore)(config, "eslint", ".eslintcache");
    yield (0, commitWithAuthor_1.commitWithAuthor)(config, "Install eslint");
}));
