"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.addTypescript = void 0;
const addScriptToPackage_1 = require("../helpers/addScriptToPackage");
const addToGitIgnore_1 = require("../helpers/addToGitIgnore");
const commitWithAuthor_1 = require("../helpers/commitWithAuthor");
const installPackage_1 = require("../helpers/installPackage");
const withStateLogger_1 = require("../helpers/withStateLogger");
const writeAndAddFile_1 = require("../helpers/writeAndAddFile");
exports.addTypescript = (0, withStateLogger_1.withStateLogger)({ id: "TypeScript" }, (config) => __awaiter(void 0, void 0, void 0, function* () {
    yield (0, installPackage_1.installPackage)(config, ["typescript@4.8.3", "@types/node@18.7.18"]);
    const tsConfigObject = {
        compilerOptions: {
            target: "ES2020",
            module: "ES2020",
            moduleResolution: "node",
            declaration: true,
            declarationMap: true,
            sourceMap: true,
            outDir: "dist",
            noEmit: true,
            isolatedModules: true,
            esModuleInterop: true,
            forceConsistentCasingInFileNames: true,
            strict: true,
            skipLibCheck: true,
            allowUnusedLabels: false,
            allowUnreachableCode: false,
            exactOptionalPropertyTypes: true,
            noFallthroughCasesInSwitch: true,
            noPropertyAccessFromIndexSignature: true,
            noUncheckedIndexedAccess: true,
            rootDir: "src",
            baseUrl: ".",
            paths: {
                "*": ["./src/*"],
            },
        },
        include: ["src/**/*"],
    };
    yield (0, writeAndAddFile_1.writeAndAddFile)(config, "tsconfig.json", JSON.stringify(tsConfigObject, null, 2));
    const tsConfigBuildObject = {
        extends: "./tsconfig.json",
        compilerOptions: {
            noEmit: false,
            sourceMap: false,
            declarationMap: false,
        },
    };
    yield (0, writeAndAddFile_1.writeAndAddFile)(config, "tsconfig.build.json", JSON.stringify(tsConfigBuildObject, null, 2));
    yield (0, addScriptToPackage_1.appendScriptToPackage)(config, "lint", "tsc --noEmit");
    yield (0, addScriptToPackage_1.appendScriptToPackage)(config, "build", "rm -rf dist && tsc -p tsconfig.build.json && resolve-tspaths -p tsconfig.build.json");
    yield (0, addScriptToPackage_1.appendScriptToPackage)(config, "prepack", "rm -rf dist && tsc -p tsconfig.build.json && resolve-tspaths -p tsconfig.build.json");
    yield (0, addToGitIgnore_1.addToGitIgnore)(config, "typescript", ["*.tsbuildinfo", "dist/"]);
    yield (0, commitWithAuthor_1.commitWithAuthor)(config, "Install typescript");
}));
