"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.setupReactComponent = void 0;
const commitWithAuthor_1 = require("../helpers/commitWithAuthor");
const installPackage_1 = require("../helpers/installPackage");
const modifyJsonFile_1 = require("../helpers/modifyJsonFile");
const withStateLogger_1 = require("../helpers/withStateLogger");
const writeAndAddFile_1 = require("../helpers/writeAndAddFile");
exports.setupReactComponent = (0, withStateLogger_1.withStateLogger)({
    id: "reactcomponent",
    message: "Configuring project as react component",
    failed: "Failed to configure as react component",
    completed: "Configured project as react component",
}, (config) => __awaiter(void 0, void 0, void 0, function* () {
    yield (0, installPackage_1.installPackage)(config, ["react", "react-dom", "@emotion/react"], { prod: true });
    yield (0, installPackage_1.installPackage)(config, [
        "@zebreus/resolve-tspaths@0.8.10",
        "@types/react",
        "@types/react-dom",
        "jest-environment-jsdom",
        "@testing-library/react",
        "@testing-library/jest-dom",
    ]);
    yield (0, installPackage_1.installPackage)(config, ["react", "react-dom"], { peer: true });
    yield (0, modifyJsonFile_1.modifyJsonConfig)(config, "tsconfig.json", tsconfigJson => (Object.assign(Object.assign({}, tsconfigJson), { compilerOptions: Object.assign(Object.assign({}, tsconfigJson.compilerOptions), { jsx: "react-jsx", jsxImportSource: "@emotion/react" }) })));
    yield (0, modifyJsonFile_1.modifyJsonConfig)(config, ".vscode/extensions.json", extensionsConfig => {
        var _a;
        return (Object.assign(Object.assign({}, extensionsConfig), { recommendations: [
                ...new Set([...((_a = extensionsConfig.recommendations) !== null && _a !== void 0 ? _a : []), "styled-components.vscode-styled-components"]),
            ] }));
    });
    yield (0, modifyJsonFile_1.modifyJsonConfig)(config, "jest.config.json", jestConfigJson => {
        var _a;
        return (Object.assign(Object.assign({}, jestConfigJson), { testEnvironment: "jsdom", setupFilesAfterEnv: [...new Set([...((_a = jestConfigJson.setupFilesAfterEnv) !== null && _a !== void 0 ? _a : []), "<rootDir>/jest-setup.ts"])] }));
    });
    yield (0, writeAndAddFile_1.writeAndAddFile)(config, "jest-setup.ts", generateJestSetup());
    yield (0, modifyJsonFile_1.modifyJsonConfig)(config, "package.json", packageJson => (Object.assign(Object.assign({}, packageJson), { files: [...new Set([...(packageJson.files || []), "dist/**"])], keywords: [...new Set([...(packageJson.keywords || []), "react", "component"])], main: "dist/index.js" })));
    yield (0, writeAndAddFile_1.writeAndAddFile)(config, "src/index.ts", generateComponentIndex());
    yield (0, writeAndAddFile_1.writeAndAddFile)(config, "src/Button.tsx", generateDemoButton());
    yield (0, writeAndAddFile_1.writeAndAddFile)(config, "src/tests/button.test.tsx", generateButtonTest());
    yield (0, commitWithAuthor_1.commitWithAuthor)(config, "Setup react component");
}));
const generateComponentIndex = () => {
    return `export { Button } from "Button"`;
};
const generateDemoButton = () => {
    return `import { css } from "@emotion/react"

export type ButtonProps = {
  text?: string
  onClick?: () => void
}
export const Button = ({ text, onClick }: ButtonProps) => (
  <button
    css={css\`
      position: relative;
      background-color: transparent;
      padding: 0.4rem 0.8rem;
      border: 2px solid #333;
      text-align: center;
      transition: all 0.35s;

      &:hover {
        color: white;
      }

      &:before {
        position: absolute;
        content: "";
        inset: 0;
        width: 0;
        background: #ff003b;
        transition: all 0.35s;
        z-index: -1;
      }

      &:hover:before {
        width: 100%;
      }
    \`}
    onClick={onClick}
  >
    {text ?? ""}
  </button>
)`;
};
const generateJestSetup = () => {
    return `// eslint-disable-next-line import/no-unassigned-import
import "@testing-library/jest-dom"`;
};
const generateButtonTest = () => {
    return `import { fireEvent, render, screen } from "@testing-library/react"
import { Button } from "Button"

it("is actually a button", () => {
  const testMessage = "Test Message"
  render(<Button text={testMessage} />)

  expect(screen.queryByText(testMessage)).toBeInstanceOf(HTMLButtonElement)
})

it("calls onClick on clicks", () => {
  const onClick = jest.fn()
  render(<Button onClick={onClick} />)

  const buttonElement = screen.queryByRole("button")
  expect(buttonElement).toBeTruthy()
  if (!buttonElement) throw new Error()

  expect(onClick).toHaveBeenCalledTimes(0)
  fireEvent.click(buttonElement)
  expect(onClick).toHaveBeenCalledTimes(1)
})`;
};
